const { GraphQLModule } = require("@graphql-modules/core");
const fs = require("fs");
const jwt = require("jsonwebtoken");
const prisma = require("../generated/prisma-client/index");

const MODULES_DIRECTORY = "./src/modules/"; // Could be an env variable

const getUser = token => {
  try {
    if(token) {
      return jwt.verify(token, process.env.JWT_SECRET);
    }

    return null;
  } catch(err) {
    return null;
  }
}

module.exports = new GraphQLModule({
    imports: load_modules()
});

function add_authorization_context(new_module) {
    new_module._options.context = ({req}) => {
        const tokenWithBearer = req.headers.authorization || '';
        const token = tokenWithBearer.split(' ')[1];
        const user = getUser(token);

        return {
            user,
            prisma
        }
    };

    return new_module;
}

function is_module_enabled(name) {
    return process.env[name.toUpperCase()] && process.env[name.toUpperCase()] === "true";
}

function load_modules() {
    let modules = [];

    let files_list = fs.readdirSync(MODULES_DIRECTORY, {withFileTypes: true});

    for(file of files_list) {
        if(file.isDirectory() && is_module_enabled(file.name)) {
            let new_module = require("./" + file.name);
            if(new_module._options.context === undefined) {
                new_module = add_authorization_context(new_module);                
            }
            
            modules.push(new_module);
        }
    }

    let installed_modules = process.env.INSTALLED_MODULES.split(',');

    for(module_name of installed_modules) {
        if(is_module_enabled(module_name)) {
            let new_module = require(module_name.toLowerCase());
            if(new_module._options.context === undefined) {
                new_module = add_authorization_context(new_module);                
            }
            
            modules.push(new_module);
        }
    }

    console.log("Loading " + modules.length + " modules");
    for(module of modules) {
        console.log("\t - " + module.name);
    }

    return modules;
}